#! /usr/bin/env python

## Copyright 2001 by LivingLogic AG, Bayreuth, Germany.
## Copyright 2001 by Walter Drwald
##
## All Rights Reserved
##
## Permission to use, copy, modify, and distribute this software and its documentation
## for any purpose and without fee is hereby granted, provided that the above copyright
## notice appears in all copies and that both that copyright notice and this permission
## notice appear in supporting documentation, and that the name of LivingLogic AG or
## the author not be used in advertising or publicity pertaining to distribution of the
## software without specific, written prior permission.
##
## LIVINGLOGIC AG AND THE AUTHOR DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
## INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
## LIVING LOGIC AG OR THE AUTHOR BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
## DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER
## IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR
## IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

r"""
<par>A module that provides some tools for file i/o related stuff.</par>

<par>The first is a function <code>forceopen</code>, that works like the
builtin function <code>open</code>, but when the directory doesn't exist
it will be created automatically. This works for nested directories too.</par>

<par>The second is a class named <code>SafeFile</code>, that encapsulates
a normal file. Nothing will be written to the file with the specified
name. All writes go to a temporary file instead. Only when the
method <code>close</code> is called, will the temporary file be closed
and the file renamed to the original name.</par>

<par>The third tool is a class <code>LogFile</code>. Writing to a log
file prepends a timestamp to each line.</par>
"""

__version__ = tuple(map(int,"$Revision: 1.4 $"[11:-2].split(".")))
# $Source: /data/cvsroot/LivingLogic/fileutils/fileutils.py,v $

import os
import os.path
import time

def forceopen(name, mode="r", buffering=-1):
	"""
	Works like the builtin function open, but is able to create
	non existing directories.
	"""
	name = os.path.expanduser(name)
	try:
		return open(name, mode, buffering)
	except IOError, e:
		if e[0] != 2: # didn't work for some other reason than a non existing directory
			raise
		(splitpath, splitname) = os.path.split(name)
		if splitpath=="": # we don't have a directory to make, so pass the error on
			raise
		os.makedirs(splitpath)
		return open(name, mode, buffering)

def remove(filename):
	"""
	removes a file that may contain user relative
	directories (i.e. ~spam/eggs)
	"""
	os.remove(os.path.expanduser(filename))

class SafeFile:
	"""
	encapsulates a normal file.
	When the file will be opened, all writes go to
	a temporary file. After all writes are finished
	to file will be renamed to its final name.
	"""

	def __init__(self, name, mode="w", buffering=0):
		self.name = name
		self.tmpname = "%s_%d.tmp" % (name, os.getpid())
		self.mode = mode
		self.buffering = buffering
		self.file = None

	def write(self, *alltexts):
		self.__open()
		for text in alltexts:
			self.file.write(text)

	def writelines(self, *alllines):
		self.__open()
		for lines in alllines:
			self.file.writelines(lines)

	def __open(self):
		if self.file is None:
			self.file = forceopen(self.tmpname, self.mode, self.buffering)

	def close(self):
		"""
		This method can be called by the user when all writes to the file
		are done. The file will be closed and moved to the final destination.
		"""
		if self.file is not None:
			self.file.close()
			os.rename(self.tmpname, self.name)
			self.file = None

class LogFile:
	"""
	A log file. All lines written to the file will be prepended with a time stamp.
	"""
	def __init__(self, name, mode="a", buffering=1, encoding="iso-8859-1"):
		"""
		__init__(filename, mode="a", buffering=1, encoding="iso-8859-1") -- construct a Log file (will be opened on the first write)
		
		filename -- the filename (this may include "~/" or "~spam/" as os.path.expanduser()
		  will be called.)
		encoding -- the encoding to use for the strings written to the file
		mode -- The mode for opening the file (should be "w" or "a")
		buffering -- the buffering for the file (0==unbuffered, 1==line, other==buffersize)
		"""
		self.startTime = time.time()
		self.name = name
		self.mode = mode
		self.buffering = buffering
		self.encoding = encoding
		self.file = None

	def formatTime(self, timestamp):
		"""
		format the timestamp into a string.
		"""
		return time.strftime("%d/%b/%Y %H:%M:%S", time.localtime(timestamp))

	def __open(self):
		if self.file is None:
			self.file = forceopen(self.name, self.mode, self.buffering)

	def write(self, *texts):
		"""
		self.log(*texts) -- write texts to the log file
		"""
		now = time.time()
		self.__open()
		for text in texts:
			lines = text.encode(self.encoding).split("\n")
			if not len(lines[-1]):
				del lines[-1]
			lines = [ "[%s]=[t+%.2fsec] %s\n" % (self.formatTime(now), now-self.startTime, line) for line in lines ]
			self.file.write("".join(lines))

